package com.malk.service.dingtalk.impl;

import cn.hutool.core.util.ObjectUtil;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDR;
import com.malk.service.dingtalk.DDClient_Contacts;
import com.malk.utils.UtilMap;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class DDImplClient_Contacts implements DDClient_Contacts {

    /**
     * 获取子部门ID列表 [获取企业部门下的所有直属子部门列表]
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/obtain-a-sub-department-id-list-v2
     */
    @Override
    public List<Number> listSubDepartmentId(String access_token, Number dept_id) {
        Map param = UtilMap.map("access_token", access_token);
        Map body = UtilMap.map("dept_id", dept_id);
        Map rsp = (Map) DDR.doPost("https://oapi.dingtalk.com/topapi/v2/department/listsubid", null, param, body).getResult();
        return (List<Number>) rsp.get("dept_id_list");
    }

    /**
     * 获取部门列表
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/obtain-the-department-list-v2
     */
    @Override
    public List<Map> listSubDepartmentDetail(String access_token, Number dept_id) {
        Map param = UtilMap.map("access_token", access_token);
        Map body = UtilMap.map("dept_id", dept_id);
        return (List<Map>) DDR.doPost("https://oapi.dingtalk.com/topapi/v2/department/listsub", null, param, body).getResult();
    }

    /**
     * 获取全部架构内全部部门
     */
    @Override
    public List<Map> getDepartmentDetail_all(String access_token) {
        return _getDepartment_all(access_token, DDConf.TOP_DEPARTMENT, new ArrayList<>(), false);
    }

    @Override
    public List<Number> getDepartmentId_all(String access_token) {
        return (List<Number>) _getDepartment_all(access_token, DDConf.TOP_DEPARTMENT, new ArrayList<>(), true)
                .stream().map(detpId -> Long.valueOf(String.valueOf(detpId))).collect(Collectors.toList());
    }

    // 递归查询
    private List _getDepartment_all(String access_token, long deptId, List deptIdList, boolean isIdList) {
        List tList = isIdList ?
                listSubDepartmentId(access_token, deptId) :
                listSubDepartmentDetail(access_token, deptId);
        deptIdList.addAll(tList);
        // 当所有子部门都为空, 递归出口
        for (Object dept : tList) {
            _getDepartment_all(access_token, Long.valueOf(String.valueOf(isIdList ? dept : ((Map) dept).get("dept_id"))), deptIdList, isIdList);
        }
        return deptIdList;
    }

    /**
     * 获取部门用户userid列表 [无需分页]
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/query-the-list-of-department-userids
     */
    @Override
    public List<String> listDepartmentUserId(String access_token, Number dept_id) {
        Map param = UtilMap.map("access_token", access_token);
        Map body = UtilMap.map("dept_id", dept_id);
        Map rsp = (Map) DDR.doPost("https://oapi.dingtalk.com/topapi/user/listid", null, param, body).getResult();
        return (List<String>) rsp.get("userid_list");
    }

    /**
     * 查询用户详情
     */
    @Override
    public Map getUserInfoById(String access_token, String userId) {
        Map param = UtilMap.map("access_token", access_token);
        Map body = UtilMap.map("userid", userId);
        return (Map) DDR.doPost("https://oapi.dingtalk.com/topapi/v2/user/get", null, param, body).getResult();
    }

    /**
     * 查询用户详情
     */
    @Override
    public Map getUserInfoByMobile(String access_token, String mobile) {
        Map param = UtilMap.map("access_token", access_token);
        Map body = UtilMap.map("mobile", mobile);
        return (Map) DDR.doPost("https://oapi.dingtalk.com/topapi/v2/user/getbymobile", null, param, body).getResult();
    }

    /**
     * 删除用户
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/delete-a-user
     */
    @Override
    public boolean deleteUser(String access_token, String userId) {
        DDR.doPost("https://oapi.dingtalk.com/topapi/v2/user/delete", null, DDConf.initTokenParams(access_token), UtilMap.map("userid", userId));
        return true;
    }

    /**
     * 创建用户
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/user-information-creation
     */
    @Override
    public Map createUser(String access_token, String name, String mobile, List<String> dept_id_list, Map body_ext) {
        Map param = UtilMap.map("access_token", access_token);
        String deptIds = String.join(",", dept_id_list.stream().map(dept -> String.valueOf(dept)).collect(Collectors.toList()));
        Map body = UtilMap.map("name, mobile, dept_id_list", name, mobile, deptIds);
        if (ObjectUtil.isNotNull(body_ext)) {
            body.putAll(body_ext);
        }
        return (Map) DDR.doPost("https://oapi.dingtalk.com/topapi/v2/user/create", null, param, body).getResult();
    }

    /**
     * 创建部门
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/create-a-department-v2
     */
    @Override
    public Map createDepartment(String access_token, String name, long parent_id, Map body_ext) {
        Map param = UtilMap.map("access_token", access_token);
        Map body = UtilMap.map("name, parent_id", name, parent_id);
        if (ObjectUtil.isNotNull(body_ext)) {
            body.putAll(body_ext);
        }
        return (Map) DDR.doPost("https://oapi.dingtalk.com/topapi/v2/department/create", null, param, body).getResult();
    }

    /**
     * 获取员工人数
     */
    @Override
    public int getUserCount(String access_token, boolean only_active) {
        Map result = (Map) DDR.doPost("https://oapi.dingtalk.com/topapi/user/count", null, DDConf.initTokenParams(access_token), UtilMap.map("only_active", only_active)).getResult();
        return UtilMap.getInt(result, "count");
    }
}
